@@ -12,10 +12,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import collections
 import contextlib
+import errno
 import functools
 import itertools
+import math
 import os
 import random
 import socket
@@ -32,8 +33,6 @@ import kombu.messaging
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import netutils
-from oslo_utils import versionutils
-import pkg_resources
 import six
 from six.moves.urllib import parse
 
@@ -100,15 +99,21 @@ rabbit_opts = [
     cfg.StrOpt('rabbit_host',
                default='localhost',
                deprecated_group='DEFAULT',
+               deprecated_for_removal=True,
+               deprecated_reason="Replaced by [DEFAULT]/transport_url",
                help='The RabbitMQ broker address where a single node is '
                     'used.'),
     cfg.PortOpt('rabbit_port',
                 default=5672,
                 deprecated_group='DEFAULT',
+                deprecated_for_removal=True,
+                deprecated_reason="Replaced by [DEFAULT]/transport_url",
                 help='The RabbitMQ broker port where a single node is used.'),
     cfg.ListOpt('rabbit_hosts',
                 default=['$rabbit_host:$rabbit_port'],
                 deprecated_group='DEFAULT',
+                deprecated_for_removal=True,
+                deprecated_reason="Replaced by [DEFAULT]/transport_url",
                 help='RabbitMQ HA cluster host:port pairs.'),
     cfg.BoolOpt('rabbit_use_ssl',
                 default=False,
@@ -117,10 +122,14 @@ rabbit_opts = [
     cfg.StrOpt('rabbit_userid',
                default='guest',
                deprecated_group='DEFAULT',
+               deprecated_for_removal=True,
+               deprecated_reason="Replaced by [DEFAULT]/transport_url",
                help='The RabbitMQ userid.'),
     cfg.StrOpt('rabbit_password',
                default='guest',
                deprecated_group='DEFAULT',
+               deprecated_for_removal=True,
+               deprecated_reason="Replaced by [DEFAULT]/transport_url",
                help='The RabbitMQ password.',
                secret=True),
     cfg.StrOpt('rabbit_login_method',
@@ -130,6 +139,8 @@ rabbit_opts = [
     cfg.StrOpt('rabbit_virtual_host',
                default='/',
                deprecated_group='DEFAULT',
+               deprecated_for_removal=True,
+               deprecated_reason="Replaced by [DEFAULT]/transport_url",
                help='The RabbitMQ virtual host.'),
     cfg.IntOpt('rabbit_retry_interval',
                default=1,
@@ -248,7 +259,7 @@ class Consumer(object):
 
     def __init__(self, exchange_name, queue_name, routing_key, type, durable,
                  exchange_auto_delete, queue_auto_delete, callback,
-                 nowait=True, rabbit_ha_queues=None, rabbit_queue_ttl=0):
+                 nowait=False, rabbit_ha_queues=None, rabbit_queue_ttl=0):
         """Init the Publisher class with the exchange_name, routing_key,
         type, durable auto_delete
         """
@@ -351,7 +362,7 @@ class ConnectionLock(DummyConnectionLock):
     starvation when the driver sends a lot of messages.
 
     So when lock.heartbeat_acquire() is called next time the lock
-    is released(), the caller unconditionnaly acquires
+    is released(), the caller unconditionally acquires
     the lock, even someone else have asked for the lock before it.
     """
 
@@ -503,10 +514,17 @@ class Connection(object):
         self._initial_pid = os.getpid()
 
         self._consumers = {}
+        self._producer = None
         self._new_tags = set()
         self._active_tags = {}
         self._tags = itertools.count(1)
 
+        # Set of exchanges and queues declared on the channel to avoid
+        # unnecessary redeclaration. This set is resetted each time
+        # the connection is resetted in Connection._set_current_channel
+        self._declared_exchanges = set()
+        self._declared_queues = set()
+
         self._consume_loop_stopped = False
         self.channel = None
         self.purpose = purpose
@@ -789,14 +807,16 @@ class Connection(object):
             return
 
         if self.channel is not None:
-            self.PUBLISHER_DECLARED_QUEUES.pop(self.channel, None)
+            self._declared_queues.clear()
+            self._declared_exchanges.clear()
             self.connection.maybe_close_channel(self.channel)
 
         self.channel = new_channel
 
-        if (new_channel is not None and
-           self.purpose == rpc_common.PURPOSE_LISTEN):
-            self._set_qos(new_channel)
+        if new_channel is not None:
+            if self.purpose == rpc_common.PURPOSE_LISTEN:
+                self._set_qos(new_channel)
+            self._producer = kombu.messaging.Producer(new_channel)
 
     def _set_qos(self, channel):
         """Set QoS prefetch count on the channel"""
@@ -868,10 +888,23 @@ class Connection(object):
             LOG.debug('Failed to get socket attribute: %s' % str(e))
         else:
             sock.settimeout(timeout)
-            if sys.platform != 'win32':
-                sock.setsockopt(socket.IPPROTO_TCP,
-                                TCP_USER_TIMEOUT,
-                                timeout * 1000 if timeout is not None else 0)
+            # TCP_USER_TIMEOUT is not defined on Windows and Mac OS X
+            if sys.platform != 'win32' and sys.platform != 'darwin':
+                try:
+                    timeout = timeout * 1000 if timeout is not None else 0
+                    # NOTE(gdavoian): only integers and strings are allowed
+                    # as socket options' values, and TCP_USER_TIMEOUT option
+                    # can take only integer values, so we round-up the timeout
+                    # to the nearest integer in order to ensure that the
+                    # connection is not broken before the expected timeout
+                    sock.setsockopt(socket.IPPROTO_TCP,
+                                    TCP_USER_TIMEOUT,
+                                    int(math.ceil(timeout)))
+                except socket.error as error:
+                    code = error[0]
+                    # TCP_USER_TIMEOUT not defined on kernels <2.6.37
+                    if code != errno.ENOPROTOOPT:
+                        raise
 
     @contextlib.contextmanager
     def _transport_socket_timeout(self, timeout):
@@ -916,7 +949,7 @@ class Connection(object):
                         # NOTE(sileht): We need to drain event to receive
                         # heartbeat from the broker but don't hold the
                         # connection too much times. In amqpdriver a connection
-                        # is used exclusivly for read or for write, so we have
+                        # is used exclusively for read or for write, so we have
                         # to do this for connection used for write drain_events
                         # already do that for other connection
                         try:
@@ -989,11 +1022,31 @@ class Connection(object):
             if not self.connection.connected:
                 raise self.connection.recoverable_connection_errors[0]
 
-            if self._new_tags:
+            consume_max_retries = 2
+            while self._new_tags and consume_max_retries:
                 for consumer, tag in self._consumers.items():
                     if tag in self._new_tags:
-                        consumer.consume(tag=tag)
-                        self._new_tags.remove(tag)
+                        try:
+                            consumer.consume(tag=tag)
+                            self._new_tags.remove(tag)
+                        except self.connection.channel_errors as exc:
+                            # NOTE(kbespalov): during the interval between
+                            # a queue declaration and consumer declaration
+                            # the queue can disappear. In this case
+                            # we must redeclare queue and try to re-consume.
+                            # More details is here:
+                            # bugs.launchpad.net/oslo.messaging/+bug/1581148
+                            if exc.code == 404 and consume_max_retries:
+                                consumer.declare(self)
+                                # NOTE(kbespalov): the broker closes a channel
+                                # at any channel error. The py-amqp catches
+                                # this situation and re-open a new channel.
+                                # So, we must re-declare all consumers again.
+                                self._new_tags = set(self._consumers.values())
+                                consume_max_retries -= 1
+                                break
+                            else:
+                                raise
 
             poll_timeout = (self._poll_timeout if timeout is None
                             else min(timeout, self._poll_timeout))
@@ -1088,24 +1141,12 @@ class Connection(object):
         with self._connection_lock:
             self.ensure(method, retry=retry, error_callback=_error_callback)
 
-    def _get_expiration(self, timeout):
-        # NOTE(gcb) kombu accept TTL as seconds instead of millisecond since
-        # version 3.0.25, so do conversion according to kombu version.
-        # TODO(gcb) remove this workaround when all supported branches
-        # with requirement kombu >=3.0.25
-        if timeout is not None:
-            kombu_version = pkg_resources.get_distribution('kombu').version
-            if not versionutils.is_compatible('3.0.25', kombu_version):
-                timeout = int(timeout * 1000)
-
-        return timeout
-
     def _publish(self, exchange, msg, routing_key=None, timeout=None):
         """Publish a message."""
-        producer = kombu.messaging.Producer(exchange=exchange,
-                                            channel=self.channel,
-                                            auto_declare=not exchange.passive,
-                                            routing_key=routing_key)
+
+        if not (exchange.passive or exchange.name in self._declared_exchanges):
+                exchange(self.channel).declare()
+                self._declared_exchanges.add(exchange.name)
 
         log_info = {'msg': msg,
                     'who': exchange or 'default',
@@ -1116,13 +1157,11 @@ class Connection(object):
         # NOTE(sileht): no need to wait more, caller expects
         # a answer before timeout is reached
         with self._transport_socket_timeout(timeout):
-            producer.publish(msg, expiration=self._get_expiration(timeout),
-                             compression=self.kombu_compression)
-
-    # List of notification queue declared on the channel to avoid
-    # unnecessary redeclaration. This list is resetted each time
-    # the connection is resetted in Connection._set_current_channel
-    PUBLISHER_DECLARED_QUEUES = collections.defaultdict(set)
+            self._producer.publish(msg,
+                                   exchange=exchange,
+                                   routing_key=routing_key,
+                                   expiration=timeout,
+                                   compression=self.kombu_compression)
 
     def _publish_and_creates_default_queue(self, exchange, msg,
                                            routing_key=None, timeout=None):
@@ -1142,8 +1181,7 @@ class Connection(object):
         # NOTE(sileht): We only do it once per reconnection
         # the Connection._set_current_channel() is responsible to clear
         # this cache
-        if (queue_indentifier not in
-                self.PUBLISHER_DECLARED_QUEUES[self.channel]):
+        if queue_indentifier not in self._declared_queues:
             queue = kombu.entity.Queue(
                 channel=self.channel,
                 exchange=exchange,
@@ -1157,7 +1195,7 @@ class Connection(object):
                 'Connection._publish_and_creates_default_queue: '
                 'declare queue %(key)s on %(exchange)s exchange', log_info)
             queue.declare()
-            self.PUBLISHER_DECLARED_QUEUES[self.channel].add(queue_indentifier)
+            self._declared_queues.add(queue_indentifier)
 
         self._publish(exchange, msg, routing_key=routing_key, timeout=timeout)
 
