@@ -40,6 +40,7 @@ _notifier_opts = [
     cfg.StrOpt('transport_url',
                deprecated_name='notification_transport_url',
                deprecated_group='DEFAULT',
+               secret=True,
                help='A URL representing the messaging driver to use for '
                     'notifications. If not set, we fall back to the same '
                     'configuration used for RPC.'),
@@ -170,7 +171,7 @@ class Notifier(object):
                       N means N retries
         :type retry: int
         :param topics: the topics which to send messages on
-        :type topic: list of strings
+        :type topics: list of strings
         """
         conf = transport.conf
         conf.register_opts(_notifier_opts,
