@@ -212,10 +212,11 @@ class TestRabbitPublisher(test_utils.BaseTestCase):
     def test_send_with_timeout(self, fake_publish):
         transport = oslo_messaging.get_transport(self.conf,
                                                  'kombu+memory:////')
+        exchange_mock = mock.Mock()
         with transport._driver._get_connection(
                 driver_common.PURPOSE_SEND) as pool_conn:
             conn = pool_conn.connection
-            conn._publish(mock.Mock(), 'msg', routing_key='routing_key',
+            conn._publish(exchange_mock, 'msg', routing_key='routing_key',
                           timeout=1)
 
         # NOTE(gcb) kombu accept TTL as seconds instead of millisecond since
@@ -226,23 +227,30 @@ class TestRabbitPublisher(test_utils.BaseTestCase):
         if versionutils.is_compatible('3.0.25', kombu_version):
             fake_publish.assert_called_with(
                 'msg', expiration=1,
-                compression=self.conf.oslo_messaging_rabbit.kombu_compression)
+                exchange=exchange_mock,
+                compression=self.conf.oslo_messaging_rabbit.kombu_compression,
+                routing_key='routing_key')
         else:
             fake_publish.assert_called_with(
                 'msg', expiration=1000,
-                compression=self.conf.oslo_messaging_rabbit.kombu_compression)
+                exchange=exchange_mock,
+                compression=self.conf.oslo_messaging_rabbit.kombu_compression,
+                routing_key='routing_key')
 
     @mock.patch('kombu.messaging.Producer.publish')
     def test_send_no_timeout(self, fake_publish):
         transport = oslo_messaging.get_transport(self.conf,
                                                  'kombu+memory:////')
+        exchange_mock = mock.Mock()
         with transport._driver._get_connection(
                 driver_common.PURPOSE_SEND) as pool_conn:
             conn = pool_conn.connection
-            conn._publish(mock.Mock(), 'msg', routing_key='routing_key')
+            conn._publish(exchange_mock, 'msg', routing_key='routing_key')
         fake_publish.assert_called_with(
             'msg', expiration=None,
-            compression=self.conf.oslo_messaging_rabbit.kombu_compression)
+            compression=self.conf.oslo_messaging_rabbit.kombu_compression,
+            exchange=exchange_mock,
+            routing_key='routing_key')
 
     def test_declared_queue_publisher(self):
         transport = oslo_messaging.get_transport(self.conf,
@@ -269,21 +277,25 @@ class TestRabbitPublisher(test_utils.BaseTestCase):
                     conn._publish_and_creates_default_queue,
                     exchange, {}, routing_key='foobar')
 
-            # Ensure the exchange does not exists
-            self.assertRaises(exc, try_send, e_passive)
-            # Create it
-            try_send(e_active)
-            # Ensure it creates it
-            try_send(e_passive)
-
-            with mock.patch('kombu.messaging.Producer', side_effect=exc):
-                # Should reset the cache and ensures the exchange does
-                # not exists
+            with mock.patch('kombu.transport.virtual.Channel.close'):
+                # Ensure the exchange does not exists
                 self.assertRaises(exc, try_send, e_passive)
-            # Recreate it
+                # Create it
+                try_send(e_active)
+                # Ensure it creates it
+                try_send(e_passive)
+
+            with mock.patch('kombu.messaging.Producer.publish',
+                            side_effect=exc):
+                # Ensure the exchange is already in cache
+                self.assertIn('foobar', conn._declared_exchanges)
+                # Reset connection
+                self.assertRaises(exc, try_send, e_passive)
+                # Ensure the cache is empty
+                self.assertEqual(0, len(conn._declared_exchanges))
+
             try_send(e_active)
-            # Ensure it have been recreated
-            try_send(e_passive)
+            self.assertIn('foobar', conn._declared_exchanges)
 
 
 class TestRabbitConsume(test_utils.BaseTestCase):
@@ -309,6 +321,51 @@ class TestRabbitConsume(test_utils.BaseTestCase):
 
         self.assertEqual(0, int(deadline - time.time()))
 
+    def test_consume_from_missing_queue(self):
+        transport = oslo_messaging.get_transport(self.conf, 'kombu+memory://')
+        self.addCleanup(transport.cleanup)
+        with transport._driver._get_connection(
+                driver_common.PURPOSE_LISTEN) as conn:
+            with mock.patch('kombu.Queue.consume') as consume, mock.patch(
+                    'kombu.Queue.declare') as declare:
+                conn.declare_topic_consumer(exchange_name='test',
+                                            topic='test',
+                                            callback=lambda msg: True)
+                import amqp
+                consume.side_effect = [amqp.NotFound, None]
+                conn.connection.connection.recoverable_connection_errors = ()
+                conn.connection.connection.recoverable_channel_errors = ()
+                self.assertEqual(1, declare.call_count)
+                conn.connection.connection.transport.drain_events = mock.Mock()
+                # Ensure that a queue will be re-declared if the consume method
+                # of kombu.Queue raise amqp.NotFound
+                conn.consume()
+                self.assertEqual(2, declare.call_count)
+
+    def test_consume_from_missing_queue_with_io_error_on_redeclaration(self):
+        transport = oslo_messaging.get_transport(self.conf, 'kombu+memory://')
+        self.addCleanup(transport.cleanup)
+        with transport._driver._get_connection(
+                driver_common.PURPOSE_LISTEN) as conn:
+            with mock.patch('kombu.Queue.consume') as consume, mock.patch(
+                    'kombu.Queue.declare') as declare:
+                conn.declare_topic_consumer(exchange_name='test',
+                                            topic='test',
+                                            callback=lambda msg: True)
+                import amqp
+                consume.side_effect = [amqp.NotFound, None]
+                declare.side_effect = [IOError, None]
+
+                conn.connection.connection.recoverable_connection_errors = (
+                    IOError,)
+                conn.connection.connection.recoverable_channel_errors = ()
+                self.assertEqual(1, declare.call_count)
+                conn.connection.connection.transport.drain_events = mock.Mock()
+                # Ensure that a queue will be re-declared after
+                # 'queue not found' exception despite on connection error.
+                conn.consume()
+                self.assertEqual(3, declare.call_count)
+
     def test_connection_ack_have_disconnected_kombu_connection(self):
         transport = oslo_messaging.get_transport(self.conf,
                                                  'kombu+memory:////')
@@ -402,11 +459,6 @@ class TestSendReceive(test_utils.BaseTestCase):
         ('zero', dict(rx_id=False, reply=0)),
     ]
 
-    _reply_fail = [
-        ('reply_success', dict(reply_failure_404=False)),
-        ('reply_failure', dict(reply_failure_404=True)),
-    ]
-
     _failure = [
         ('success', dict(failure=False)),
         ('failure', dict(failure=True, expected=False)),
@@ -423,7 +475,6 @@ class TestSendReceive(test_utils.BaseTestCase):
         cls.scenarios = testscenarios.multiply_scenarios(cls._n_senders,
                                                          cls._context,
                                                          cls._reply,
-                                                         cls._reply_fail,
                                                          cls._failure,
                                                          cls._timeout)
 
@@ -440,26 +491,16 @@ class TestSendReceive(test_utils.BaseTestCase):
 
         target = oslo_messaging.Target(topic='testtopic')
 
-        listener = driver.listen(target)
+        listener = driver.listen(target, None, None)._poll_style_listener
 
         senders = []
         replies = []
         msgs = []
-        errors = []
-
-        def stub_error(msg, *a, **kw):
-            if (a and len(a) == 1 and isinstance(a[0], dict) and a[0]):
-                a = a[0]
-            errors.append(str(msg) % a)
-
-        self.stubs.Set(driver_common.LOG, 'error', stub_error)
 
         def send_and_wait_for_reply(i):
             try:
-                if self.reply_failure_404:
-                    timeout = 0.01
-                else:
-                    timeout = self.timeout
+
+                timeout = self.timeout
                 replies.append(driver.send(target,
                                            self.ctxt,
                                            {'tx_id': i},
@@ -469,8 +510,7 @@ class TestSendReceive(test_utils.BaseTestCase):
                 self.assertIsNone(self.timeout)
             except (ZeroDivisionError, oslo_messaging.MessagingTimeout) as e:
                 replies.append(e)
-                self.assertTrue(self.failure or self.timeout is not None
-                                or self.reply_failure_404)
+                self.assertTrue(self.failure or self.timeout is not None)
 
         while len(senders) < self.n_senders:
             senders.append(threading.Thread(target=send_and_wait_for_reply,
@@ -490,18 +530,6 @@ class TestSendReceive(test_utils.BaseTestCase):
         if len(order) > 1:
             order[-1], order[-2] = order[-2], order[-1]
 
-        if self.reply_failure_404:
-            start = time.time()
-            # NOTE(sileht): Simulate a rpc client restart
-            # By returning a ExchangeNotFound when we try to
-            # send reply
-            exc = (driver._reply_q_conn.connection.
-                   connection.channel_errors[0]())
-            exc.code = 404
-            self.useFixture(mockpatch.Patch(
-                'kombu.messaging.Producer.publish',
-                side_effect=exc))
-
         for i in order:
             if self.timeout is None:
                 if self.failure:
@@ -509,28 +537,16 @@ class TestSendReceive(test_utils.BaseTestCase):
                         raise ZeroDivisionError
                     except Exception:
                         failure = sys.exc_info()
-                    msgs[i].reply(failure=failure,
-                                  log_failure=not self.expected)
+                    msgs[i].reply(failure=failure)
                 elif self.rx_id:
                     msgs[i].reply({'rx_id': i})
                 else:
                     msgs[i].reply(self.reply)
-            elif self.reply_failure_404:
-                msgs[i].reply({})
             senders[i].join()
 
-        if self.reply_failure_404:
-            # NOTE(sileht) all reply fail, first take
-            # kombu_missing_consumer_retry_timeout seconds to fail
-            # next immediately fail
-            dt = time.time() - start
-            rabbit_conf = self.conf.oslo_messaging_rabbit
-            timeout = rabbit_conf.kombu_missing_consumer_retry_timeout
-            self.assertTrue(timeout <= dt < (timeout + 0.100), dt)
-
         self.assertEqual(len(senders), len(replies))
         for i, reply in enumerate(replies):
-            if self.timeout is not None or self.reply_failure_404:
+            if self.timeout is not None:
                 self.assertIsInstance(reply, oslo_messaging.MessagingTimeout)
             elif self.failure:
                 self.assertIsInstance(reply, ZeroDivisionError)
@@ -539,11 +555,6 @@ class TestSendReceive(test_utils.BaseTestCase):
             else:
                 self.assertEqual(self.reply, reply)
 
-        if not self.timeout and self.failure and not self.expected:
-            self.assertTrue(len(errors) > 0, errors)
-        else:
-            self.assertEqual(0, len(errors), errors)
-
 
 TestSendReceive.generate_scenarios()
 
@@ -556,7 +567,7 @@ class TestPollAsync(test_utils.BaseTestCase):
         self.addCleanup(transport.cleanup)
         driver = transport._driver
         target = oslo_messaging.Target(topic='testtopic')
-        listener = driver.listen(target)
+        listener = driver.listen(target, None, None)._poll_style_listener
         received = listener.poll(timeout=0.050)
         self.assertEqual([], received)
 
@@ -572,8 +583,7 @@ class TestRacyWaitForReply(test_utils.BaseTestCase):
 
         target = oslo_messaging.Target(topic='testtopic')
 
-        listener = driver.listen(target)
-
+        listener = driver.listen(target, None, None)._poll_style_listener
         senders = []
         replies = []
         msgs = []
@@ -909,7 +919,7 @@ class TestReplyWireFormat(test_utils.BaseTestCase):
                                        server=self.server,
                                        fanout=self.fanout)
 
-        listener = driver.listen(target)
+        listener = driver.listen(target, None, None)._poll_style_listener
 
         connection, producer = _create_producer(target)
         self.addCleanup(connection.release)
