@@ -290,6 +290,12 @@ class SkipIfNoTransportURL(test_utils.BaseTestCase):
         zmq_redis_port = os.environ.get('ZMQ_REDIS_PORT')
         if zmq_redis_port:
             self.config(port=zmq_redis_port, group="matchmaker_redis")
+        zmq_use_pub_sub = os.environ.get('ZMQ_USE_PUB_SUB')
+        if zmq_use_pub_sub:
+            self.config(use_pub_sub=zmq_use_pub_sub)
+        zmq_use_router_proxy = os.environ.get('ZMQ_USE_ROUTER_PROXY')
+        if zmq_use_router_proxy:
+            self.config(use_router_proxy=zmq_use_router_proxy)
 
 
 class NotificationFixture(fixtures.Fixture):
