@@ -27,6 +27,8 @@ __all__ = [
     'set_transport_defaults',
 ]
 
+import logging
+
 from oslo_config import cfg
 import six
 from six.moves.urllib import parse
@@ -34,16 +36,20 @@ from stevedore import driver
 
 from oslo_messaging import exceptions
 
+LOG = logging.getLogger(__name__)
 
 _transport_opts = [
     cfg.StrOpt('transport_url',
+               secret=True,
                help='A URL representing the messaging driver to use and its '
-                    'full configuration. If not set, we fall back to the '
-                    'rpc_backend option and driver specific configuration.'),
+                    'full configuration.'),
     cfg.StrOpt('rpc_backend',
+               deprecated_for_removal=True,
+               deprecated_reason="Replaced by [DEFAULT]/transport_url",
                default='rabbit',
                help='The messaging driver to use, defaults to rabbit. Other '
                     'drivers include amqp and zmq.'),
+
     cfg.StrOpt('control_exchange',
                default='openstack',
                help='The default exchange under which topics are scoped. May '
@@ -96,21 +102,24 @@ class Transport(object):
         self._driver.send_notification(target, ctxt, message, version,
                                        retry=retry)
 
-    def _listen(self, target):
+    def _listen(self, target, batch_size, batch_timeout):
         if not (target.topic and target.server):
             raise exceptions.InvalidTarget('A server\'s target must have '
                                            'topic and server names specified',
                                            target)
-        return self._driver.listen(target)
+        return self._driver.listen(target, batch_size,
+                                   batch_timeout)
 
-    def _listen_for_notifications(self, targets_and_priorities, pool):
+    def _listen_for_notifications(self, targets_and_priorities, pool,
+                                  batch_size, batch_timeout):
         for target, priority in targets_and_priorities:
             if not target.topic:
                 raise exceptions.InvalidTarget('A target must have '
                                                'topic specified',
                                                target)
         return self._driver.listen_for_notifications(
-            targets_and_priorities, pool)
+            targets_and_priorities, pool, batch_size, batch_timeout
+        )
 
     def cleanup(self):
         """Release all resources associated with this transport."""
@@ -169,11 +178,7 @@ def get_transport(conf, url=None, allowed_remote_exmods=None, aliases=None):
     conf.register_opts(_transport_opts)
 
     if not isinstance(url, TransportURL):
-        url = url or conf.transport_url
-        parsed = TransportURL.parse(conf, url, aliases)
-        if not parsed.transport:
-            raise InvalidTransportURL(url, 'No scheme specified in "%s"' % url)
-        url = parsed
+        url = TransportURL.parse(conf, url, aliases)
 
     kwargs = dict(default_exchange=conf.control_exchange,
                   allowed_remote_exmods=allowed_remote_exmods)
@@ -225,10 +230,12 @@ class TransportURL(object):
 
     Transport URLs take the form::
 
-      transport://user:pass@host1:port[,hostN:portN]/virtual_host
+      transport://user:pass@host:port[,userN:passN@hostN:portN]/virtual_host?query
 
     i.e. the scheme selects the transport driver, you may include multiple
-    hosts in netloc and the path part is a "virtual host" partition path.
+    hosts in netloc, the path part is a "virtual host" partition path and
+    the query part contains some driver-specific options which may override
+    corresponding values from a static configuration.
 
     :param conf: a ConfigOpts instance
     :type conf: oslo.config.cfg.ConfigOpts
@@ -238,12 +245,14 @@ class TransportURL(object):
     :type virtual_host: str
     :param hosts: a list of TransportHost objects
     :type hosts: list
-    :param aliases: A map of transport alias to transport name
+    :param aliases: DEPRECATED: a map of transport alias to transport name
     :type aliases: dict
+    :param query: a dictionary of URL query parameters
+    :type query: dict
     """
 
     def __init__(self, conf, transport=None, virtual_host=None, hosts=None,
-                 aliases=None):
+                 aliases=None, query=None):
         self.conf = conf
         self.conf.register_opts(_transport_opts)
         self._transport = transport
@@ -256,6 +265,12 @@ class TransportURL(object):
             self.aliases = {}
         else:
             self.aliases = aliases
+        if query is None:
+            self.query = {}
+        else:
+            self.query = query
+
+        self._deprecation_logged = False
 
     @property
     def transport(self):
@@ -263,7 +278,20 @@ class TransportURL(object):
             transport = self.conf.rpc_backend
         else:
             transport = self._transport
-        return self.aliases.get(transport, transport)
+        final_transport = self.aliases.get(transport, transport)
+        if not self._deprecation_logged and final_transport != transport:
+            # NOTE(sileht): The first step is deprecate this one cycle.
+            # To ensure deployer have updated they configuration during Octavia
+            # Then in P we will deprecate aliases kwargs of TransportURL() and
+            # get_transport() for consuming application
+            LOG.warning('legacy "rpc_backend" is deprecated, '
+                        '"%(legacy_transport)s" must be replaced by '
+                        '"%(final_transport)s"' % {
+                            'legacy_transport': transport,
+                            'final_transport': final_transport})
+            self._deprecation_logged = True
+
+        return final_transport
 
     @transport.setter
     def transport(self, value):
@@ -326,15 +354,18 @@ class TransportURL(object):
         if self.virtual_host:
             url += parse.quote(self.virtual_host)
 
+        if self.query:
+            url += '?' + parse.urlencode(self.query, doseq=True)
+
         return url
 
     @classmethod
-    def parse(cls, conf, url, aliases=None):
+    def parse(cls, conf, url=None, aliases=None):
         """Parse an url.
 
         Assuming a URL takes the form of::
 
-          transport://user:pass@host1:port[,hostN:portN]/virtual_host
+          transport://user:pass@host:port[,userN:passN@hostN:portN]/virtual_host?query
 
         then parse the URL and return a TransportURL object.
 
@@ -351,6 +382,8 @@ class TransportURL(object):
               {"host": "host2:port2"}
             ]
 
+        If the url is not provided conf.transport_url is parsed instead.
+
         :param conf: a ConfigOpts instance
         :type conf: oslo.config.cfg.ConfigOpts
         :param url: The URL to parse
@@ -359,6 +392,8 @@ class TransportURL(object):
         :type aliases: dict
         :returns: A TransportURL
         """
+
+        url = url or conf.transport_url
         if not url:
             return cls(conf, aliases=aliases)
 
@@ -367,17 +402,19 @@ class TransportURL(object):
 
         url = parse.urlparse(url)
 
-        # Make sure there's not a query string; that could identify
-        # requirements we can't comply with (for example ssl), so reject it if
-        # it's present
-        if '?' in url.path or url.query:
-            raise InvalidTransportURL(url.geturl(),
-                                      "Cannot comply with query string in "
-                                      "transport URL")
+        if not url.scheme:
+            raise InvalidTransportURL(url.geturl(), 'No scheme specified')
+
+        transport = url.scheme
+
+        query = {}
+        if url.query:
+            for key, values in six.iteritems(parse.parse_qs(url.query)):
+                query[key] = ','.join(values)
 
         virtual_host = None
         if url.path.startswith('/'):
-            virtual_host = url.path[1:]
+            virtual_host = parse.unquote(url.path[1:])
 
         hosts = []
 
@@ -392,6 +429,8 @@ class TransportURL(object):
                 username, hostname = host.split('@', 1)
                 if ':' in username:
                     username, password = username.split(':', 1)
+                    password = parse.unquote(password)
+                username = parse.unquote(username)
 
             if not hostname:
                 hostname = None
@@ -401,7 +440,7 @@ class TransportURL(object):
                 if host_end < 0:
                     # NOTE(Vek): Identical to what Python 2.7's
                     # urlparse.urlparse() raises in this case
-                    raise ValueError("Invalid IPv6 URL")
+                    raise ValueError('Invalid IPv6 URL')
 
                 port_text = hostname[host_end:]
                 hostname = hostname[1:host_end]
@@ -420,4 +459,4 @@ class TransportURL(object):
                                        username=username,
                                        password=password))
 
-        return cls(conf, url.scheme, virtual_host, hosts, aliases)
+        return cls(conf, transport, virtual_host, hosts, aliases, query)
